// generated from file '../src/faust/gxamp4.dsp' by dsp2cc:
// Code generated with Faust (https://faust.grame.fr)

#include "valve.h"

namespace gxamp4 {

class Dsp: public PluginDef {
private:
	gx_resample::FixedRateResampler smp;
	int sample_rate;
	int fSampleRate;
	FAUSTFLOAT fCheckbox0;
	FAUSTFLOAT	*fCheckbox0_;
	double fConst0;
	double fConst1;
	double fConst2;
	double fConst3;
	FAUSTFLOAT fVslider0;
	FAUSTFLOAT	*fVslider0_;
	double fRec4[2];
	FAUSTFLOAT fVslider1;
	FAUSTFLOAT	*fVslider1_;
	double fRec6[2];
	FAUSTFLOAT fVslider2;
	FAUSTFLOAT	*fVslider2_;
	double fRec7[2];
	FAUSTFLOAT fCheckbox1;
	FAUSTFLOAT	*fCheckbox1_;
	FAUSTFLOAT fVslider3;
	FAUSTFLOAT	*fVslider3_;
	double fRec21[2];
	double fVec0[2];
	double fRec20[2];
	double fRec19[3];
	double fConst4;
	double fConst5;
	double fVec1[2];
	double fConst6;
	double fRec22[2];
	double fRec18[2];
	double fConst7;
	double fConst8;
	double fRec17[2];
	double fConst9;
	double fConst10;
	double fConst11;
	double fRec23[2];
	double fVec2[2];
	double fConst12;
	double fConst13;
	double fConst14;
	double fRec16[2];
	double fRec15[2];
	double fRec14[3];
	double fConst15;
	double fConst16;
	double fVec3[2];
	double fConst17;
	double fRec24[2];
	double fRec13[2];
	double fRec12[2];
	double fRec25[2];
	double fVec4[2];
	double fRec11[2];
	double fRec10[3];
	double fConst18;
	double fConst19;
	double fVec5[2];
	double fConst20;
	double fRec26[2];
	double fRec9[2];
	double fRec8[2];
	double fRec27[2];
	double fConst21;
	double fConst22;
	double fConst23;
	double fConst24;
	double fConst25;
	double fConst26;
	double fConst27;
	double fConst28;
	double fConst29;
	double fConst30;
	double fConst31;
	double fConst32;
	double fConst33;
	double fConst34;
	double fConst35;
	double fConst36;
	double fConst37;
	double fConst38;
	double fConst39;
	double fConst40;
	double fConst41;
	double fConst42;
	double fConst43;
	double fConst44;
	double fConst45;
	double fConst46;
	double fConst47;
	double fRec36[3];
	double fVec6[2];
	double fConst48;
	double fRec35[2];
	double fConst49;
	double fConst50;
	double fRec34[2];
	double fRec33[3];
	double fVec7[2];
	double fConst51;
	double fConst52;
	double fConst53;
	double fRec32[2];
	double fRec31[3];
	double fVec8[2];
	double fConst54;
	double fConst55;
	double fConst56;
	double fRec30[2];
	double fRec29[3];
	double fConst57;
	double fRec38[2];
	double fRec37[3];
	double fConst58;
	double fConst59;
	double fConst60;
	double fConst61;
	double fConst62;
	double fRec42[2];
	double fRec41[3];
	double fConst63;
	double fRec40[3];
	double fRec39[3];
	double fConst64;
	double fRec45[2];
	double fRec44[3];
	double fConst65;
	double fRec43[3];
	double fVec9[2];
	double fRec28[2];
	double fVec10[2];
	double fRec5[2];
	double fVec11[2];
	double fRec3[2];
	double fRec2[3];
	double fVec12[2];
	double fRec46[2];
	double fRec1[2];
	double fConst66;
	double fRec0[2];
	double fRec47[2];
	double fVec13[2];
	double fRec50[2];
	double fRec49[2];
	double fRec48[2];
	double fRec51[2];

	void clear_state_f();
	void init(unsigned int sample_rate);
	void compute(int count, FAUSTFLOAT *input0, FAUSTFLOAT *output0);
	int register_par(const ParamReg& reg);

	static void clear_state_f_static(PluginDef*);
	static void init_static(unsigned int sample_rate, PluginDef*);
	static void compute_static(int count, FAUSTFLOAT *input0, FAUSTFLOAT *output0, PluginDef*);
	static int register_params_static(const ParamReg& reg);
	static void del_instance(PluginDef *p);
public:
	Dsp();
	~Dsp();
};



Dsp::Dsp()
	: PluginDef() {
	version = PLUGINDEF_VERSION;
	flags = 0;
	id = "pre 12AU7/ master 6V6";
	name = N_("pre 12AU7/ master 6V6");
	groups = 0;
	description = ""; // description (tooltip)
	category = "";       // category
	shortname = "";     // shortname
	mono_audio = compute_static;
	stereo_audio = 0;
	set_samplerate = init_static;
	activate_plugin = 0;
	register_params = register_params_static;
	load_ui = 0;
	clear_state = clear_state_f_static;
	delete_instance = del_instance;
}

Dsp::~Dsp() {
}

inline void Dsp::clear_state_f()
{
	for (int l0 = 0; l0 < 2; l0 = l0 + 1) fRec4[l0] = 0.0;
	for (int l1 = 0; l1 < 2; l1 = l1 + 1) fRec6[l1] = 0.0;
	for (int l2 = 0; l2 < 2; l2 = l2 + 1) fRec7[l2] = 0.0;
	for (int l3 = 0; l3 < 2; l3 = l3 + 1) fRec21[l3] = 0.0;
	for (int l4 = 0; l4 < 2; l4 = l4 + 1) fVec0[l4] = 0.0;
	for (int l5 = 0; l5 < 2; l5 = l5 + 1) fRec20[l5] = 0.0;
	for (int l6 = 0; l6 < 3; l6 = l6 + 1) fRec19[l6] = 0.0;
	for (int l7 = 0; l7 < 2; l7 = l7 + 1) fVec1[l7] = 0.0;
	for (int l8 = 0; l8 < 2; l8 = l8 + 1) fRec22[l8] = 0.0;
	for (int l9 = 0; l9 < 2; l9 = l9 + 1) fRec18[l9] = 0.0;
	for (int l10 = 0; l10 < 2; l10 = l10 + 1) fRec17[l10] = 0.0;
	for (int l11 = 0; l11 < 2; l11 = l11 + 1) fRec23[l11] = 0.0;
	for (int l12 = 0; l12 < 2; l12 = l12 + 1) fVec2[l12] = 0.0;
	for (int l13 = 0; l13 < 2; l13 = l13 + 1) fRec16[l13] = 0.0;
	for (int l14 = 0; l14 < 2; l14 = l14 + 1) fRec15[l14] = 0.0;
	for (int l15 = 0; l15 < 3; l15 = l15 + 1) fRec14[l15] = 0.0;
	for (int l16 = 0; l16 < 2; l16 = l16 + 1) fVec3[l16] = 0.0;
	for (int l17 = 0; l17 < 2; l17 = l17 + 1) fRec24[l17] = 0.0;
	for (int l18 = 0; l18 < 2; l18 = l18 + 1) fRec13[l18] = 0.0;
	for (int l19 = 0; l19 < 2; l19 = l19 + 1) fRec12[l19] = 0.0;
	for (int l20 = 0; l20 < 2; l20 = l20 + 1) fRec25[l20] = 0.0;
	for (int l21 = 0; l21 < 2; l21 = l21 + 1) fVec4[l21] = 0.0;
	for (int l22 = 0; l22 < 2; l22 = l22 + 1) fRec11[l22] = 0.0;
	for (int l23 = 0; l23 < 3; l23 = l23 + 1) fRec10[l23] = 0.0;
	for (int l24 = 0; l24 < 2; l24 = l24 + 1) fVec5[l24] = 0.0;
	for (int l25 = 0; l25 < 2; l25 = l25 + 1) fRec26[l25] = 0.0;
	for (int l26 = 0; l26 < 2; l26 = l26 + 1) fRec9[l26] = 0.0;
	for (int l27 = 0; l27 < 2; l27 = l27 + 1) fRec8[l27] = 0.0;
	for (int l28 = 0; l28 < 2; l28 = l28 + 1) fRec27[l28] = 0.0;
	for (int l29 = 0; l29 < 3; l29 = l29 + 1) fRec36[l29] = 0.0;
	for (int l30 = 0; l30 < 2; l30 = l30 + 1) fVec6[l30] = 0.0;
	for (int l31 = 0; l31 < 2; l31 = l31 + 1) fRec35[l31] = 0.0;
	for (int l32 = 0; l32 < 2; l32 = l32 + 1) fRec34[l32] = 0.0;
	for (int l33 = 0; l33 < 3; l33 = l33 + 1) fRec33[l33] = 0.0;
	for (int l34 = 0; l34 < 2; l34 = l34 + 1) fVec7[l34] = 0.0;
	for (int l35 = 0; l35 < 2; l35 = l35 + 1) fRec32[l35] = 0.0;
	for (int l36 = 0; l36 < 3; l36 = l36 + 1) fRec31[l36] = 0.0;
	for (int l37 = 0; l37 < 2; l37 = l37 + 1) fVec8[l37] = 0.0;
	for (int l38 = 0; l38 < 2; l38 = l38 + 1) fRec30[l38] = 0.0;
	for (int l39 = 0; l39 < 3; l39 = l39 + 1) fRec29[l39] = 0.0;
	for (int l40 = 0; l40 < 2; l40 = l40 + 1) fRec38[l40] = 0.0;
	for (int l41 = 0; l41 < 3; l41 = l41 + 1) fRec37[l41] = 0.0;
	for (int l42 = 0; l42 < 2; l42 = l42 + 1) fRec42[l42] = 0.0;
	for (int l43 = 0; l43 < 3; l43 = l43 + 1) fRec41[l43] = 0.0;
	for (int l44 = 0; l44 < 3; l44 = l44 + 1) fRec40[l44] = 0.0;
	for (int l45 = 0; l45 < 3; l45 = l45 + 1) fRec39[l45] = 0.0;
	for (int l46 = 0; l46 < 2; l46 = l46 + 1) fRec45[l46] = 0.0;
	for (int l47 = 0; l47 < 3; l47 = l47 + 1) fRec44[l47] = 0.0;
	for (int l48 = 0; l48 < 3; l48 = l48 + 1) fRec43[l48] = 0.0;
	for (int l49 = 0; l49 < 2; l49 = l49 + 1) fVec9[l49] = 0.0;
	for (int l50 = 0; l50 < 2; l50 = l50 + 1) fRec28[l50] = 0.0;
	for (int l51 = 0; l51 < 2; l51 = l51 + 1) fVec10[l51] = 0.0;
	for (int l52 = 0; l52 < 2; l52 = l52 + 1) fRec5[l52] = 0.0;
	for (int l53 = 0; l53 < 2; l53 = l53 + 1) fVec11[l53] = 0.0;
	for (int l54 = 0; l54 < 2; l54 = l54 + 1) fRec3[l54] = 0.0;
	for (int l55 = 0; l55 < 3; l55 = l55 + 1) fRec2[l55] = 0.0;
	for (int l56 = 0; l56 < 2; l56 = l56 + 1) fVec12[l56] = 0.0;
	for (int l57 = 0; l57 < 2; l57 = l57 + 1) fRec46[l57] = 0.0;
	for (int l58 = 0; l58 < 2; l58 = l58 + 1) fRec1[l58] = 0.0;
	for (int l59 = 0; l59 < 2; l59 = l59 + 1) fRec0[l59] = 0.0;
	for (int l60 = 0; l60 < 2; l60 = l60 + 1) fRec47[l60] = 0.0;
	for (int l61 = 0; l61 < 2; l61 = l61 + 1) fVec13[l61] = 0.0;
	for (int l62 = 0; l62 < 2; l62 = l62 + 1) fRec50[l62] = 0.0;
	for (int l63 = 0; l63 < 2; l63 = l63 + 1) fRec49[l63] = 0.0;
	for (int l64 = 0; l64 < 2; l64 = l64 + 1) fRec48[l64] = 0.0;
	for (int l65 = 0; l65 < 2; l65 = l65 + 1) fRec51[l65] = 0.0;
}

void Dsp::clear_state_f_static(PluginDef *p)
{
	static_cast<Dsp*>(p)->clear_state_f();
}

inline void Dsp::init(unsigned int RsamplingFreq)
{
	sample_rate = 96000;
	smp.setup(RsamplingFreq, sample_rate);
	fSampleRate = sample_rate;
	fConst0 = std::min<double>(1.92e+05, std::max<double>(1.0, double(fSampleRate)));
	fConst1 = std::tan(97.38937226128358 / fConst0);
	fConst2 = 1.0 / fConst1;
	fConst3 = 1.0 - fConst2;
	fConst4 = 1.0 / std::tan(270.1769682087222 / fConst0);
	fConst5 = 1.0 - fConst4;
	fConst6 = 1.0 / (fConst4 + 1.0);
	fConst7 = 0.1 / fConst1;
	fConst8 = 1.0 / (fConst2 + 1.0);
	fConst9 = 3.141592653589793 / fConst0;
	fConst10 = 1.0 - fConst9;
	fConst11 = 1.0 / (fConst9 + 1.0);
	fConst12 = 1.0 / std::tan(20517.741620594938 / fConst0);
	fConst13 = 1.0 - fConst12;
	fConst14 = 1.0 / (fConst12 + 1.0);
	fConst15 = 1.0 / std::tan(414.6902302738527 / fConst0);
	fConst16 = 1.0 - fConst15;
	fConst17 = 1.0 / (fConst15 + 1.0);
	fConst18 = 1.0 / std::tan(609.4689747964198 / fConst0);
	fConst19 = 1.0 - fConst18;
	fConst20 = 1.0 / (fConst18 + 1.0);
	fConst21 = std::tan(942.4777960769379 / fConst0);
	fConst22 = mydsp_faustpower2_f(fConst21);
	fConst23 = 2.0 * (1.0 - 1.0 / fConst22);
	fConst24 = 1.0 / fConst21;
	fConst25 = (fConst24 + -1.0000000000000004) / fConst21 + 1.0;
	fConst26 = (fConst24 + 1.0000000000000004) / fConst21 + 1.0;
	fConst27 = 1.0 / fConst26;
	fConst28 = std::tan(3769.9111843077517 / fConst0);
	fConst29 = mydsp_faustpower2_f(fConst28);
	fConst30 = 2.0 * (1.0 - 1.0 / fConst29);
	fConst31 = 1.0 / fConst28;
	fConst32 = (fConst31 + -1.0000000000000004) / fConst28 + 1.0;
	fConst33 = (fConst31 + 1.0000000000000004) / fConst28 + 1.0;
	fConst34 = 1.0 / fConst33;
	fConst35 = std::tan(10053.096491487338 / fConst0);
	fConst36 = mydsp_faustpower2_f(fConst35);
	fConst37 = 2.0 * (1.0 - 1.0 / fConst36);
	fConst38 = 1.0 / fConst35;
	fConst39 = (fConst38 + -1.0000000000000004) / fConst35 + 1.0;
	fConst40 = (fConst38 + 1.0000000000000004) / fConst35 + 1.0;
	fConst41 = 1.0 / fConst40;
	fConst42 = std::tan(47123.8898038469 / fConst0);
	fConst43 = 2.0 * (1.0 - 1.0 / mydsp_faustpower2_f(fConst42));
	fConst44 = 1.0 / fConst42;
	fConst45 = (fConst44 + -1.414213562373095) / fConst42 + 1.0;
	fConst46 = (fConst44 + 1.414213562373095) / fConst42 + 1.0;
	fConst47 = 1.0 / fConst46;
	fConst48 = 1.0 / (fConst1 * fConst46);
	fConst49 = 1.0 - fConst38;
	fConst50 = 1.0 / (fConst38 + 1.0);
	fConst51 = 1.0 - fConst31;
	fConst52 = fConst31 + 1.0;
	fConst53 = 1.0 / fConst52;
	fConst54 = 1.0 - fConst24;
	fConst55 = fConst24 + 1.0;
	fConst56 = 1.0 / fConst55;
	fConst57 = 1.0 / (fConst21 * fConst33);
	fConst58 = 1.0 / (fConst22 * fConst26);
	fConst59 = 1.0 - fConst54 / fConst21;
	fConst60 = 1.0 / (fConst55 / fConst21 + 1.0);
	fConst61 = 1.0 - fConst51 / fConst28;
	fConst62 = 1.0 / (fConst52 / fConst28 + 1.0);
	fConst63 = 1.0 / (fConst36 * fConst40);
	fConst64 = 1.0 / (fConst28 * fConst40);
	fConst65 = 1.0 / (fConst29 * fConst33);
	fConst66 = 0.025 / fConst1;
	clear_state_f();
}

void Dsp::init_static(unsigned int sample_rate, PluginDef *p)
{
	static_cast<Dsp*>(p)->init(sample_rate);
}

void always_inline Dsp::compute(int count, FAUSTFLOAT *input0, FAUSTFLOAT *output0)
{
#define fCheckbox0 (*fCheckbox0_)
#define fVslider0 (*fVslider0_)
#define fVslider1 (*fVslider1_)
#define fVslider2 (*fVslider2_)
#define fCheckbox1 (*fCheckbox1_)
#define fVslider3 (*fVslider3_)
	FAUSTFLOAT buf[smp.max_out_count(count)];
	int ReCount = smp.up(count, input0, buf);
	int iSlow0 = int(double(fCheckbox0));
	double fSlow1 = 0.0010000000000000009 * std::pow(1e+01, 0.05 * double(fVslider0));
	double fSlow2 = 0.0010000000000000009 * double(fVslider1);
	double fSlow3 = 1.000000000000001e-05 * double(fVslider2);
	int iSlow4 = int(double(fCheckbox1));
	double fSlow5 = 0.0010000000000000009 * std::pow(1e+01, 0.05 * double(fVslider3));
	for (int i0 = 0; i0 < ReCount; i0 = i0 + 1) {
		fRec4[0] = fSlow1 + 0.999 * fRec4[1];
		fRec6[0] = fSlow2 + 0.999 * fRec6[1];
		double fTemp0 = 2.0 * (fRec6[0] + -0.5);
		fRec7[0] = fSlow3 + 0.999 * fRec7[1];
		double fTemp1 = 1.0 - fRec7[0];
		fRec21[0] = fSlow5 + 0.999 * fRec21[1];
		double fTemp2 = double(buf[i0]) * fRec21[0];
		fVec0[0] = fTemp2;
		fRec20[0] = 0.9302847925323914 * (fTemp2 + fVec0[1]) - 0.8605695850647829 * fRec20[1];
		fRec19[0] = fRec20[0] - (1.8405051250752198 * fRec19[1] + 0.8612942439318627 * fRec19[2]);
		double fTemp3 = 1.8508996845035413 * fRec19[1];
		double fTemp4 = 0.9254498422517706 * (fRec19[0] + fRec19[2]);
		double fTemp5 = (fTemp3 + fTemp4) / (Ranode(TUBE_TABLE_12AU7_68k, fRec18[1]) + 1e+05);
		fVec1[0] = fTemp5;
		fRec22[0] = fConst6 * (2.7e+03 * (fTemp5 + fVec1[1]) - fConst5 * fRec22[1]);
		fRec18[0] = Ftube(TUBE_TABLE_12AU7_68k, fTemp4 + fRec22[0] + fTemp3 + -1.25724) + -83.43555555555557;
		double fTemp6 = fRec18[0] - fRec18[1];
		fRec17[0] = fConst8 * (fConst7 * fTemp6 - fConst3 * fRec17[1]);
		fRec23[0] = fConst11 * (fConst10 * fRec23[1] + 0.1 * fTemp6);
		double fTemp7 = ((iSlow4) ? fRec23[0] : fRec17[0]);
		fVec2[0] = fTemp7;
		fRec16[0] = -(fConst14 * (fConst13 * fRec16[1] - (fTemp7 + fVec2[1])));
		fRec15[0] = 0.9302847925323914 * (fRec16[0] + fRec16[1]) - 0.8605695850647829 * fRec15[1];
		fRec14[0] = fRec15[0] - (1.8405051250752198 * fRec14[1] + 0.8612942439318627 * fRec14[2]);
		double fTemp8 = 1.8508996845035413 * fRec14[1];
		double fTemp9 = 0.9254498422517706 * (fRec14[0] + fRec14[2]);
		double fTemp10 = (fTemp8 + fTemp9) / (Ranode(TUBE_TABLE_12AU7_250k, fRec13[1]) + 1e+05);
		fVec3[0] = fTemp10;
		fRec24[0] = fConst17 * (1.5e+03 * (fTemp10 + fVec3[1]) - fConst16 * fRec24[1]);
		fRec13[0] = Ftube(TUBE_TABLE_12AU7_250k, fTemp9 + fRec24[0] + fTemp8 + -0.776162) + -78.25586666666666;
		double fTemp11 = fRec13[0] - fRec13[1];
		fRec12[0] = fConst8 * (fConst7 * fTemp11 - fConst3 * fRec12[1]);
		fRec25[0] = fConst11 * (fConst10 * fRec25[1] + 0.1 * fTemp11);
		double fTemp12 = ((iSlow4) ? fRec25[0] : fRec12[0]);
		fVec4[0] = fTemp12;
		fRec11[0] = 0.9302847925323914 * (fTemp12 + fVec4[1]) - 0.8605695850647829 * fRec11[1];
		fRec10[0] = fRec11[0] - (1.8405051250752198 * fRec10[1] + 0.8612942439318627 * fRec10[2]);
		double fTemp13 = 1.8508996845035413 * fRec10[1];
		double fTemp14 = 0.9254498422517706 * (fRec10[0] + fRec10[2]);
		double fTemp15 = (fTemp13 + fTemp14) / (Ranode(TUBE_TABLE_12AU7_250k, fRec9[1]) + 1e+05);
		fVec5[0] = fTemp15;
		fRec26[0] = fConst20 * (8.2e+02 * (fTemp15 + fVec5[1]) - fConst19 * fRec26[1]);
		fRec9[0] = Ftube(TUBE_TABLE_12AU7_250k, fTemp14 + fRec26[0] + fTemp13 + -0.445487) + -75.67231707317073;
		double fTemp16 = fRec9[0] - fRec9[1];
		fRec8[0] = fConst8 * (fConst7 * fTemp16 - fConst3 * fRec8[1]);
		fRec27[0] = fConst11 * (fConst10 * fRec27[1] + 0.1 * fTemp16);
		double fTemp17 = ((iSlow4) ? fRec27[0] : fRec8[0]);
		fRec36[0] = fRec7[0] * fTemp17 - fConst47 * (fConst45 * fRec36[2] + fConst43 * fRec36[1]);
		double fTemp18 = fRec36[2] + fRec36[0] + 2.0 * fRec36[1];
		fVec6[0] = fTemp18;
		fRec35[0] = -(fConst8 * (fConst3 * fRec35[1] - fConst48 * (fTemp18 - fVec6[1])));
		fRec34[0] = -(fConst50 * (fConst49 * fRec34[1] - (fRec35[0] + fRec35[1])));
		fRec33[0] = fRec34[0] - fConst41 * (fConst39 * fRec33[2] + fConst37 * fRec33[1]);
		double fTemp19 = fRec33[2] + fRec33[0] + 2.0 * fRec33[1];
		fVec7[0] = fTemp19;
		fRec32[0] = -(fConst53 * (fConst51 * fRec32[1] - fConst41 * (fTemp19 + fVec7[1])));
		fRec31[0] = fRec32[0] - fConst34 * (fConst32 * fRec31[2] + fConst30 * fRec31[1]);
		double fTemp20 = fRec31[2] + fRec31[0] + 2.0 * fRec31[1];
		fVec8[0] = fTemp20;
		fRec30[0] = -(fConst56 * (fConst54 * fRec30[1] - fConst34 * (fTemp20 + fVec8[1])));
		fRec29[0] = fRec30[0] - fConst27 * (fConst25 * fRec29[2] + fConst23 * fRec29[1]);
		double fTemp21 = std::max<double>(-1.0, std::min<double>(1.0, fConst27 * (fRec29[2] + fRec29[0] + 2.0 * fRec29[1]) * std::pow(1e+01, 0.9 * fRec6[0])));
		fRec38[0] = -(fConst56 * (fConst54 * fRec38[1] - fConst57 * (fTemp20 - fVec8[1])));
		fRec37[0] = fRec38[0] - fConst27 * (fConst25 * fRec37[2] + fConst23 * fRec37[1]);
		double fTemp22 = std::max<double>(-1.0, std::min<double>(1.0, fConst58 * (fRec37[2] + (fRec37[0] - 2.0 * fRec37[1])) * std::pow(1e+01, 2.0 * fRec6[0])));
		double fTemp23 = fConst23 * fRec39[1];
		double fTemp24 = fConst30 * fRec40[1];
		fRec42[0] = -(fConst50 * (fConst49 * fRec42[1] - fConst38 * (fRec35[0] - fRec35[1])));
		fRec41[0] = fRec42[0] - fConst41 * (fConst39 * fRec41[2] + fConst37 * fRec41[1]);
		fRec40[0] = fConst63 * (fRec41[2] + (fRec41[0] - 2.0 * fRec41[1])) - fConst62 * (fConst61 * fRec40[2] + fTemp24);
		fRec39[0] = fRec40[2] + fConst62 * (fTemp24 + fConst61 * fRec40[0]) - fConst60 * (fConst59 * fRec39[2] + fTemp23);
		double fTemp25 = std::max<double>(-1.0, std::min<double>(1.0, (fRec39[2] + fConst60 * (fTemp23 + fConst59 * fRec39[0])) * std::pow(1e+01, 0.8 * fRec6[0])));
		double fTemp26 = fConst23 * fRec43[1];
		fRec45[0] = -(fConst53 * (fConst51 * fRec45[1] - fConst64 * (fTemp19 - fVec7[1])));
		fRec44[0] = fRec45[0] - fConst34 * (fConst32 * fRec44[2] + fConst30 * fRec44[1]);
		fRec43[0] = fConst65 * (fRec44[2] + (fRec44[0] - 2.0 * fRec44[1])) - fConst60 * (fConst59 * fRec43[2] + fTemp26);
		double fTemp27 = std::max<double>(-1.0, std::min<double>(1.0, (fRec43[2] + fConst60 * (fTemp26 + fConst59 * fRec43[0])) * std::pow(1e+01, 1.2 * fRec6[0])));
		double fTemp28 = 1.333521432 * fTemp27 * (1.0 - 0.3333333333333333 * mydsp_faustpower2_f(fTemp27)) + 1.584893192 * (fTemp25 * (1.0 - 0.3333333333333333 * mydsp_faustpower2_f(fTemp25)) + fTemp22 * (1.0 - 0.3333333333333333 * mydsp_faustpower2_f(fTemp22))) + 1.2589412 * fTemp21 * (1.0 - 0.3333333333333333 * mydsp_faustpower2_f(fTemp21));
		fVec9[0] = fTemp28;
		fRec28[0] = -(fConst14 * (fConst13 * fRec28[1] - (fTemp28 + fVec9[1])));
		double fTemp29 = fRec28[0] * (1.0 - std::max<double>(0.0, -fTemp0)) + fTemp17 * (fTemp1 * (1.25 * fRec6[0] * (4.0 * (1.0 - std::fabs(fTemp17 * fTemp1)) + -1.0) + 1.0) + fRec7[0] * (1.0 - std::max<double>(0.0, fTemp0)));
		fVec10[0] = fTemp29;
		fRec5[0] = -(fConst14 * (fConst13 * fRec5[1] - (fTemp29 + fVec10[1])));
		double fTemp30 = fRec5[0] * fRec4[0];
		fVec11[0] = fTemp30;
		fRec3[0] = 0.9302847925323914 * (fTemp30 + fVec11[1]) - 0.8605695850647829 * fRec3[1];
		fRec2[0] = fRec3[0] - (1.8405051250752198 * fRec2[1] + 0.8612942439318627 * fRec2[2]);
		double fTemp31 = 1.8508996845035413 * fRec2[1];
		double fTemp32 = 0.9254498422517706 * (fRec2[0] + fRec2[2]);
		double fTemp33 = fTemp31 + fTemp32;
		double fTemp34 = fTemp33 / (Ranode(TUBE_TABLE_6V6_250k, fRec1[1]) + 1e+05);
		fVec12[0] = fTemp34;
		fRec46[0] = fConst14 * (8.2e+02 * (fTemp34 + fVec12[1]) - fConst13 * fRec46[1]);
		fRec1[0] = Ftube(TUBE_TABLE_6V6_250k, fTemp32 + fRec46[0] + fTemp31 + -0.962132) + -132.6668292682927;
		double fTemp35 = fRec1[0] - fRec1[1];
		fRec0[0] = fConst8 * (fConst66 * fTemp35 - fConst3 * fRec0[1]);
		fRec47[0] = fConst11 * (fConst10 * fRec47[1] + 0.025 * fTemp35);
		double fTemp36 = fTemp33 / (Ranode(TUBE_TABLE_6V6_68k, fRec49[1]) + 1e+05);
		fVec13[0] = fTemp36;
		fRec50[0] = fConst14 * (8.2e+02 * (fTemp36 + fVec13[1]) - fConst13 * fRec50[1]);
		fRec49[0] = Ftube(TUBE_TABLE_6V6_68k, fTemp32 + fRec50[0] + fTemp31 + -0.963342) + -132.5192682926829;
		double fTemp37 = fRec49[0] - fRec49[1];
		fRec48[0] = fConst8 * (fConst66 * fTemp37 - fConst3 * fRec48[1]);
		fRec51[0] = fConst11 * (fConst10 * fRec51[1] + 0.025 * fTemp37);
		buf[i0] = FAUSTFLOAT(((iSlow0) ? fRec51[0] : fRec48[0]) + ((iSlow0) ? fRec47[0] : fRec0[0]));
		fRec4[1] = fRec4[0];
		fRec6[1] = fRec6[0];
		fRec7[1] = fRec7[0];
		fRec21[1] = fRec21[0];
		fVec0[1] = fVec0[0];
		fRec20[1] = fRec20[0];
		fRec19[2] = fRec19[1];
		fRec19[1] = fRec19[0];
		fVec1[1] = fVec1[0];
		fRec22[1] = fRec22[0];
		fRec18[1] = fRec18[0];
		fRec17[1] = fRec17[0];
		fRec23[1] = fRec23[0];
		fVec2[1] = fVec2[0];
		fRec16[1] = fRec16[0];
		fRec15[1] = fRec15[0];
		fRec14[2] = fRec14[1];
		fRec14[1] = fRec14[0];
		fVec3[1] = fVec3[0];
		fRec24[1] = fRec24[0];
		fRec13[1] = fRec13[0];
		fRec12[1] = fRec12[0];
		fRec25[1] = fRec25[0];
		fVec4[1] = fVec4[0];
		fRec11[1] = fRec11[0];
		fRec10[2] = fRec10[1];
		fRec10[1] = fRec10[0];
		fVec5[1] = fVec5[0];
		fRec26[1] = fRec26[0];
		fRec9[1] = fRec9[0];
		fRec8[1] = fRec8[0];
		fRec27[1] = fRec27[0];
		fRec36[2] = fRec36[1];
		fRec36[1] = fRec36[0];
		fVec6[1] = fVec6[0];
		fRec35[1] = fRec35[0];
		fRec34[1] = fRec34[0];
		fRec33[2] = fRec33[1];
		fRec33[1] = fRec33[0];
		fVec7[1] = fVec7[0];
		fRec32[1] = fRec32[0];
		fRec31[2] = fRec31[1];
		fRec31[1] = fRec31[0];
		fVec8[1] = fVec8[0];
		fRec30[1] = fRec30[0];
		fRec29[2] = fRec29[1];
		fRec29[1] = fRec29[0];
		fRec38[1] = fRec38[0];
		fRec37[2] = fRec37[1];
		fRec37[1] = fRec37[0];
		fRec42[1] = fRec42[0];
		fRec41[2] = fRec41[1];
		fRec41[1] = fRec41[0];
		fRec40[2] = fRec40[1];
		fRec40[1] = fRec40[0];
		fRec39[2] = fRec39[1];
		fRec39[1] = fRec39[0];
		fRec45[1] = fRec45[0];
		fRec44[2] = fRec44[1];
		fRec44[1] = fRec44[0];
		fRec43[2] = fRec43[1];
		fRec43[1] = fRec43[0];
		fVec9[1] = fVec9[0];
		fRec28[1] = fRec28[0];
		fVec10[1] = fVec10[0];
		fRec5[1] = fRec5[0];
		fVec11[1] = fVec11[0];
		fRec3[1] = fRec3[0];
		fRec2[2] = fRec2[1];
		fRec2[1] = fRec2[0];
		fVec12[1] = fVec12[0];
		fRec46[1] = fRec46[0];
		fRec1[1] = fRec1[0];
		fRec0[1] = fRec0[0];
		fRec47[1] = fRec47[0];
		fVec13[1] = fVec13[0];
		fRec50[1] = fRec50[0];
		fRec49[1] = fRec49[0];
		fRec48[1] = fRec48[0];
		fRec51[1] = fRec51[0];
	}
	smp.down(buf, output0);
#undef fCheckbox0
#undef fVslider0
#undef fVslider1
#undef fVslider2
#undef fCheckbox1
#undef fVslider3
}

void __rt_func Dsp::compute_static(int count, FAUSTFLOAT *input0, FAUSTFLOAT *output0, PluginDef *p)
{
	static_cast<Dsp*>(p)->compute(count, input0, output0);
}

int Dsp::register_par(const ParamReg& reg)
{
	fVslider1_ = reg.registerFloatVar("gxdistortion.drive","","SA","",&fVslider1, 0.35, 0.0, 1.0, 0.01, 0);
	fVslider2_ = reg.registerFloatVar("gxdistortion.wet_dry","","SA","",&fVslider2, 1e+02, 0.0, 1e+02, 1.0, 0);
	fCheckbox1_ = reg.registerFloatVar("amp.highgain","","BA",N_("Allow frequencies below 31Hz"),&fCheckbox1, 0.0, 0.0, 1.0, 1.0, 0);
	fVslider3_ = reg.registerFloatVar("amp2.stage1.Pregain","","SA","",&fVslider3, -6.0, -2e+01, 2e+01, 0.1, 0);
	fCheckbox0_ = reg.registerFloatVar("amp.highgain","","BA",N_("Allow frequencies below 31Hz"),&fCheckbox0, 0.0, 0.0, 1.0, 1.0, 0);
	fVslider0_ = reg.registerFloatVar("amp2.stage2.gain1","","SA","",&fVslider0, -6.0, -2e+01, 2e+01, 0.1, 0);
	return 0;
}

int Dsp::register_params_static(const ParamReg& reg)
{
	return static_cast<Dsp*>(reg.plugin)->register_par(reg);
}

PluginDef *plugin() {
	return new Dsp();
}

void Dsp::del_instance(PluginDef *p)
{
	delete static_cast<Dsp*>(p);
}

} // end namespace gxamp4
