// generated from file '../src/faust/drumseq.dsp' by dsp2cc:
// Code generated with Faust (https://faust.grame.fr)

#define FAUSTFLOAT float

namespace drumseq {

class Dsp {
private:
	int fSampleRate;
	FAUSTFLOAT fCheckbox0;
	FAUSTFLOAT	*fCheckbox0_;
	int iVec0[2];
	FAUSTFLOAT fVslider0;
	double fRec0[2];
	FAUSTFLOAT fVslider1;
	double fRec1[2];
	double fConst0;
	double fConst1;
	double fConst2;
	double fConst3;
	double fConst4;
	double fConst5;
	double fConst6;
	double fConst7;
	double fConst8;
	int iRec4[2];
	double fVec1[2];
	double fConst9;
	double fConst10;
	double fRec3[2];
	double fRec2[3];
	double fConst11;
	FAUSTFLOAT fCheckbox1;
	FAUSTFLOAT	*fCheckbox1_;
	double fVec2[2];
	double fRec7[2];
	int iRec5[2];
	double fConst12;
	double fConst13;
	double fConst14;
	double fRec6[2];
	double fConst15;
	double fConst16;
	double fConst17;
	double fConst18;
	double fConst19;
	double fConst20;
	double fConst21;
	double fConst22;
	double fRec9[2];
	double fRec8[3];
	int iRec10[2];
	double fConst23;
	double fRec11[2];
	double fConst24;
	double fConst25;
	double fConst26;
	double fRec12[2];
	double fRec13[2];
	int iRec14[2];
	double fConst27;
	double fRec15[2];
	double fConst28;
	double fConst29;
	double fConst30;
	double fRec16[2];
	double fRec17[2];
	int iRec18[2];
	double fConst31;
	double fRec19[2];
	FAUSTFLOAT fVslider2;
	double fRec20[2];
	double fConst32;
	FAUSTFLOAT fCheckbox2;
	FAUSTFLOAT	*fCheckbox2_;
	double fVec3[2];
	double fRec21[2];
	double fConst33;
	double fConst34;
	double fRec26[2];
	double fVec4[2];
	int IOTA0;
	double fVec5[64];
	double fConst35;
	int iConst36;
	int iConst37;
	double fConst38;
	double fConst39;
	double fConst40;
	double fConst41;
	double fConst42;
	double fConst43;
	double fRec27[2];
	double fRec28[2];
	double fConst44;
	double fConst45;
	double fRec24[2];
	double fRec25[2];
	double fConst46;
	double fConst47;
	double fConst48;
	double fRec31[2];
	double fRec32[2];
	double fConst49;
	double fRec29[2];
	double fRec30[2];
	double fConst50;
	double fConst51;
	double fConst52;
	double fRec35[2];
	double fRec36[2];
	double fConst53;
	double fRec33[2];
	double fRec34[2];
	double fConst54;
	double fConst55;
	double fConst56;
	double fRec39[2];
	double fRec40[2];
	double fConst57;
	double fRec37[2];
	double fRec38[2];
	double fVec6[2];
	double fRec23[2];
	double fRec22[3];
	double fConst58;
	double fRec43[2];
	int iRec41[2];
	double fConst59;
	double fConst60;
	double fConst61;
	double fRec42[2];
	double fConst62;
	double fRec45[2];
	double fRec44[3];
	int iRec46[2];
	double fConst63;
	double fRec47[2];
	FAUSTFLOAT fVslider3;
	FAUSTFLOAT	*fVslider3_;
	double fRec48[2];
	FAUSTFLOAT fCheckbox3;
	FAUSTFLOAT	*fCheckbox3_;
	double fVec7[2];
	double fRec51[2];
	int iRec49[2];
	double fRec50[2];
	FAUSTFLOAT fVslider4;
	FAUSTFLOAT	*fVslider4_;
	double fRec52[2];
	double fRec53[2];
	double fConst64;
	double fRec54[2];
	double fRec55[2];
	double fConst65;
	double fRec58[2];
	double fRec59[2];
	double fConst66;
	double fRec60[2];
	double fRec56[2];
	double fRec57[2];
	int iRec62[2];
	double fConst67;
	double fRec63[2];
	FAUSTFLOAT fVslider5;
	double fRec64[2];
	double fConst68;
	FAUSTFLOAT fCheckbox4;
	FAUSTFLOAT	*fCheckbox4_;
	double fVec8[2];
	double fRec65[2];
	double fConst69;
	double fConst70;
	double fConst71;
	double fConst72;
	double fConst73;
	double fConst74;
	double fConst75;
	double fConst76;
	double fConst77;
	double fConst78;
	double fRec69[2];
	double fRec68[3];
	double fVec9[2];
	double fConst79;
	double fRec67[2];
	double fRec66[3];
	double fRec72[2];
	int iRec70[2];
	double fConst80;
	double fRec71[2];
	double fConst81;
	double fConst82;
	double fConst83;
	double fConst84;
	double fConst85;
	double fConst86;
	double fRec77[2];
	double fConst87;
	double fConst88;
	double fConst89;
	double fRec78[2];
	double fRec79[2];
	double fRec75[2];
	double fRec76[2];
	double fConst90;
	double fConst91;
	double fRec74[2];
	double fRec73[3];
	double fRec82[2];
	int iRec80[2];
	double fRec81[2];


public:
	void clear_state_f();
	void init(unsigned int sample_rate);
	void compute(int count, FAUSTFLOAT *input0, FAUSTFLOAT *output0);
	int register_par(const ParamReg& reg);
	Dsp();
	~Dsp();
};

} // end namespace drumseq
