// Generated by gmmproc 2.66.7 -- DO NOT MODIFY!
#ifndef _GXWMM_CONTROLPARAMETER_P_H
#define _GXWMM_CONTROLPARAMETER_P_H


#include <glibmm/private/interface_p.h>

#include <glibmm/private/interface_p.h>

namespace Gxw
{

class  ControlParameter_Class : public Glib::Interface_Class
{
public:
  using CppObjectType = ControlParameter;
  using BaseObjectType = GxControlParameter;
  using BaseClassType = GxControlParameterIface;
  using CppClassParent = Glib::Interface_Class;

  friend class ControlParameter;

  const Glib::Interface_Class& init();

  static void iface_init_function(void* g_iface, void* iface_data);

  static Glib::ObjectBase* wrap_new(GObject*);

protected:

  //Callbacks (default signal handlers):
  //These will call the *_impl member methods, which will then call the existing default signal callbacks, if any.
  //You could prevent the original default signal handlers being called by overriding the *_impl method.

  //Callbacks (virtual functions):
};


} // namespace Gxw


#endif /* _GXWMM_CONTROLPARAMETER_P_H */

