// Generated by gmmproc 2.66.7 -- DO NOT MODIFY!


#include <glibmm.h>

#include <gxwmm/toggleimage.h>
#include <gxwmm/private/toggleimage_p.h>


/*
 * Copyright (C) 2009, 2010 Hermann Meyer, James Warden, Andreas Degert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gxw/GxToggleImage.h>

namespace
{
} // anonymous namespace


namespace Glib
{

Gxw::ToggleImage* wrap(GxToggleImage* object, bool take_copy)
{
  return dynamic_cast<Gxw::ToggleImage *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gxw
{


/* The *_Class implementation: */

const Glib::Class& ToggleImage_Class::init()
{
  if(!gtype_) // create the GType if necessary
  {
    // Glib::Class has to know the class init function to clone custom types.
    class_init_func_ = &ToggleImage_Class::class_init_function;

    // This is actually just optimized away, apparently with no harm.
    // Make sure that the parent type has been created.
    //CppClassParent::CppObjectType::get_type();

    // Create the wrapper type, with the same class/instance size as the base type.
    register_derived_type(gx_toggle_image_get_type());

    // Add derived versions of interfaces, if the C type implements any interfaces:

  }

  return *this;
}


void ToggleImage_Class::class_init_function(void* g_class, void* class_data)
{
  const auto klass = static_cast<BaseClassType*>(g_class);
  CppClassParent::class_init_function(klass, class_data);


}


Glib::ObjectBase* ToggleImage_Class::wrap_new(GObject* o)
{
  return manage(new ToggleImage((GxToggleImage*)(o)));

}


/* The implementation: */

ToggleImage::ToggleImage(const Glib::ConstructParams& construct_params)
:
  Gtk::Misc(construct_params)
{
  }

ToggleImage::ToggleImage(GxToggleImage* castitem)
:
  Gtk::Misc((GtkMisc*)(castitem))
{
  }


ToggleImage::ToggleImage(ToggleImage&& src) noexcept
: Gtk::Misc(std::move(src))
{}

ToggleImage& ToggleImage::operator=(ToggleImage&& src) noexcept
{
  Gtk::Misc::operator=(std::move(src));
  return *this;
}

ToggleImage::~ToggleImage() noexcept
{
  destroy_();
}

ToggleImage::CppClassType ToggleImage::toggleimage_class_; // initialize static member

GType ToggleImage::get_type()
{
  return toggleimage_class_.init().get_type();
}


GType ToggleImage::get_base_type()
{
  return gx_toggle_image_get_type();
}


ToggleImage::ToggleImage()
:
  // Mark this class as non-derived to allow C++ vfuncs to be skipped.
  Glib::ObjectBase(nullptr),
  Gtk::Misc(Glib::ConstructParams(toggleimage_class_.init()))
{
  

}

void ToggleImage::set_base_name(const Glib::ustring& p1)
{
  gx_toggle_image_set_base_name(gobj(), p1.c_str());
}

Glib::ustring ToggleImage::get_base_name()
{
  return Glib::convert_const_gchar_ptr_to_ustring(gx_toggle_image_get_base_name(gobj()));
}


Glib::PropertyProxy< Glib::ustring > ToggleImage::property_base_name() 
{
  return Glib::PropertyProxy< Glib::ustring >(this, "base-name");
}

Glib::PropertyProxy_ReadOnly< Glib::ustring > ToggleImage::property_base_name() const
{
  return Glib::PropertyProxy_ReadOnly< Glib::ustring >(this, "base-name");
}

Glib::PropertyProxy< Glib::ustring > ToggleImage::property_var_id() 
{
  return Glib::PropertyProxy< Glib::ustring >(this, "var-id");
}

Glib::PropertyProxy_ReadOnly< Glib::ustring > ToggleImage::property_var_id() const
{
  return Glib::PropertyProxy_ReadOnly< Glib::ustring >(this, "var-id");
}


} // namespace Gxw


