#include "gx_plugin.h"

namespace pluginlib {
namespace zita_rev1 { PluginDef *plugin(); }
namespace ts9sim { PluginDef *plugin(); }
namespace flanger_gx { PluginDef *plugin(); }
namespace mbc { PluginDef *plugin(); }
namespace mbcs { PluginDef *plugin(); }
namespace mbd { PluginDef *plugin(); }
namespace mbe { PluginDef *plugin(); }
namespace mbdel { PluginDef *plugin(); }
namespace mbchor { PluginDef *plugin(); }
namespace mbclipper { PluginDef *plugin(); }
namespace gcb_95 { PluginDef *plugin(); }
namespace fuzzfacerm { PluginDef *plugin(); }
namespace fuzzfacefm { PluginDef *plugin(); }
namespace hfb { PluginDef *plugin(); }
namespace muff { PluginDef *plugin(); }
namespace susta { PluginDef *plugin(); }
namespace scream { PluginDef *plugin(); }
namespace hornet { PluginDef *plugin(); }
namespace foxeylady { PluginDef *plugin(); }
namespace lpbboost { PluginDef *plugin(); }
namespace hogsfoot { PluginDef *plugin(); }
namespace cstb { PluginDef *plugin(); }
namespace coverd { PluginDef *plugin(); }
namespace rangem { PluginDef *plugin(); }
namespace ruin { PluginDef *plugin(); }
namespace ffreak { PluginDef *plugin(); }
namespace fumaster { PluginDef *plugin(); }
namespace fuzzdrive { PluginDef *plugin(); }
namespace rolandwah { PluginDef *plugin(); }
namespace astrofuzz { PluginDef *plugin(); }
namespace buffb { PluginDef *plugin(); }
namespace eldist { PluginDef *plugin(); }
namespace mole { PluginDef *plugin(); }
namespace buzz { PluginDef *plugin(); }
namespace bfuzz { PluginDef *plugin(); }
namespace axface { PluginDef *plugin(); }
namespace aclipper { PluginDef *plugin(); }
namespace bmp { PluginDef *plugin(); }
namespace mxrdist { PluginDef *plugin(); }
namespace bossds1 { PluginDef *plugin(); }
namespace bassboom { PluginDef *plugin(); }
namespace vibe { PluginDef *plugin_mono(); }
namespace vibe { PluginDef *plugin_stereo(); }
namespace metronome { PluginDef *plugin(); }
namespace abgate { PluginDef *plugin(); }
namespace jcm800pre { PluginDef *plugin(); }
namespace reversedelay { PluginDef *plugin(); }
namespace vumeter { PluginDef *plugin(); }
namespace vumeter_st { PluginDef *plugin(); }
}
